@@ -13,7 +13,7 @@
 ##############################################################################
 
 """
-$Id: tests.py 114890 2010-07-21 12:18:55Z adamg $
+$Id: tests.py 115479 2010-08-05 08:19:23Z adamg $
 """
 __docformat__ = 'restructuredtext'
 
@@ -142,6 +142,16 @@ class NegotiatorTest(zope.component.testing.PlacelessSetup,
         self.negotiator.clearCache(env)
         self.negotiator.clearCache(env)
 
+        #edge case, cache has a more specific language than available
+        self.negotiator.policy = 'browser'
+        env = testing.EnvStub(('de-de','de'))
+
+        self.assertEqual(self.negotiator.getLanguage(('de-de', 'de'), env), 'de-de')
+
+        self.assertEqual(self.negotiator.getLanguage(('de', 'en'), env), 'de')
+
+        self.negotiator.clearCache(env)
+
 
 def test_suite():
     return unittest.TestSuite((
